#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::HTTP;
######################################################################

use Cognition::MagicString;
use Cognition::uF;
use DateTime;
use URI::URL;
use XML::LibXML qw(:all);
use strict;


sub is_enabled
{
	my $this = shift;
	
	return 1
		if ($this->{options}->{p_http} && $this->{RESPONSE});
		
	return 0;
}


sub consume
{
	my $this = shift;
	return unless ($this->{RESPONSE});
	my $nsrdf   = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
	my $nshttp  = 'http://www.w3.org/2006/http#';
	my $nshttpH = 'http://www.w3.org/2008/http-headers#';
	my $nshttpM = 'http://www.w3.org/2008/http-methods#';
	my $nshttpS = 'http://www.w3.org/2008/http-statusCodes#';
	my $rdftype = $nsrdf.'type';

	my $Response = Cognition::RDFModel::Subject->new(
		Cognition::HTMLParser::Utils::BNodeURI(undef, 'HttpResponse'));
	$Response->addVal($rdftype, Cognition::RDFModel::Value->new($nshttp.'Response', 1));
	my $ResponseH = Cognition::RDFModel::Subject->new(
		Cognition::HTMLParser::Utils::BNodeURI(undef, 'HttpResponseHeaders'));
	$ResponseH->addVal($rdftype, Cognition::RDFModel::Value->new($nsrdf.'List', 1));
	$Response->addVal($nshttp.'headers',
		Cognition::RDFModel::Value->new($ResponseH->uri, 1));

	my $Request = Cognition::RDFModel::Subject->new(
		Cognition::HTMLParser::Utils::BNodeURI(undef, 'HttpRequest'));
	$Request->addVal($rdftype, Cognition::RDFModel::Value->new($nshttp.'Request', 1));
	my $RequestH = Cognition::RDFModel::Subject->new(
		Cognition::HTMLParser::Utils::BNodeURI(undef, 'HttpRequestHeaders'));
	$RequestH->addVal($rdftype, Cognition::RDFModel::Value->new($nsrdf.'List', 1));
	$Request->addVal($nshttp.'headers',
		Cognition::RDFModel::Value->new($RequestH->uri, 1));
	$Request->addVal($nshttp.'response',
		Cognition::RDFModel::Value->new($Response->uri, 1));
	
	my $Subject = Cognition::RDFModel::Subject->new($this->uri);
	$Subject->addVal('http://buzzword.org.uk/rdf/http-ext#related-request',
		Cognition::RDFModel::Value->new($Request->uri, 1));

	my $i = 1;
	
	foreach my $header ( $this->{REQUEST}->headers->header_field_names )
	{
		my @h = $this->{REQUEST}->header($header);
		foreach my $h (@h)
		{
			my $huri = consume_header($this, $Subject, $header, $h);
			$RequestH->addVal('http://buzzword.org.uk/rdf/list-items/_'.$i,
				Cognition::RDFModel::Value->new($huri, 1));
			$i++;
		}
	}
	
	my $rproto = $this->{REQUEST}->protocol;
	$rproto =~ s|^HTTP/||;
	
	$Request->addVal($nshttp.'httpVersion',
		Cognition::RDFModel::Value->new($rproto))
		if ($rproto);
	$Request->addVal($nshttp.'method',
		Cognition::RDFModel::Value->new($nshttpM.uc($this->{REQUEST}->method), 1));
	$Request->addVal($nshttp.'methodName',
		Cognition::RDFModel::Value->new($this->{REQUEST}->method));
	my $val = Cognition::RDFModel::Value->new($this->{REQUEST}->uri);
	$val->setDatatype('http://www.w3.org/2001/XMLSchema#anyURI');
	$Request->addVal($nshttp.'absoluteURI', $val);
	
	$i = 1;
	foreach my $header ( $this->{RESPONSE}->headers->header_field_names )
	{
		my @h = $this->{RESPONSE}->header($header);
		foreach my $h (@h)
		{
			my $huri = consume_header($this, $Subject, $header, $h);
			$ResponseH->addVal('http://buzzword.org.uk/rdf/list-items/_'.$i,
				Cognition::RDFModel::Value->new($huri, 1));
			$i++;
		}
	}
	
	my $rdate = DateTime->from_epoch( epoch => $this->{RESPONSE}->headers->date);
	$rproto = $this->{RESPONSE}->protocol;
	$rproto =~ s|^HTTP/||;
	
	$val = Cognition::RDFModel::Value->new($rproto);
	$val->setDatatype('http://www.w3.org/2001/XMLSchema#float');
	$Response->addVal($nshttp.'httpVersion', $val) if ($rproto);
	$Response->addVal($nshttp.'statusCode',
		Cognition::RDFModel::Value->new($nshttpS.'statusCode'.$this->{RESPONSE}->code, 1));
	$val = Cognition::RDFModel::Value->new($this->{RESPONSE}->code);
	$val->setDatatype('http://www.w3.org/2001/XMLSchema#decimal');
	$Response->addVal($nshttp.'statusCodeNumber', $val);
	$Response->addVal($nshttp.'reasonPhrase',
		Cognition::RDFModel::Value->new($this->{RESPONSE}->message));
	$Response->addVal('http://purl.org/dc/terms/date',
		Cognition::RDFModel::Value->new($rdate));
	
	$this->mergeSubject($Subject);
	$this->mergeSubject($Request);
	$this->mergeSubject($RequestH);
	$this->mergeSubject($Response);
	$this->mergeSubject($ResponseH);
	
	$this->mergeNS('ianalink',
		'http://www.iana.org/assignments/relation/',
		'IANA Registered Link Types',
		-1,
		0);
	$this->mergeNS('http',
		$nshttp,
		'HTTP Vocabulary in RDF',
		-1,
		0);
	$this->mergeNS('http-ext',
		'http://buzzword.org.uk/rdf/http-ext#',
		'HTTP Vocabulary in RDF Extentions',
		-1,
		0);
	$this->mergeNS('httphdr',
		$nshttpH,
		'HTTP Vocabulary in RDF (Headers)',
		-1,
		0);
	$this->mergeNS('httpmeth',
		$nshttpM,
		'HTTP Vocabulary in RDF (Methods)',
		-1,
		0);
	$this->mergeNS('httpstat',
		$nshttpS,
		'HTTP Vocabulary in RDF (Status Codes)',
		-1,
		0);
}

sub relationship
{
	my $str   = shift;

	if ($str =~ /^([a-z][a-z0-9\+\.\-]*)\:/i)
	{
		# seems to be an absolute URI, so can safely return "as is".
		return $str;
	}

	return 'http://www.iana.org/assignments/relation/'.lc($str);

	my $url = url lc($str), 'http://www.iana.org/assignments/relation/';
	return $url->abs->as_string;

	return undef;
}

sub consume_header
{
	my $nsrdf   = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
	my $nshttp  = 'http://www.w3.org/2006/http#';
	my $nshttpH = 'http://www.w3.org/2008/http-headers#';
	my $nshttpM = 'http://www.w3.org/2008/http-methods#';
	my $nshttpS = 'http://www.w3.org/2008/http-statusCodes#';
	my $rdftype = $nsrdf.'type';

	my $this    = shift;
	my $Subject = shift;
	my $header  = shift;
	my $value   = shift;
	
	my $Header = Cognition::RDFModel::Subject->new(
		Cognition::HTMLParser::Utils::BNodeURI(undef, 'MessageHeader'));
	$Header->addVal($rdftype,
		Cognition::RDFModel::Value->new($nshttp.'MessageHeader', 1));

	$Header->addVal($nshttp.'fieldName',
		Cognition::RDFModel::Value->new($header));
	$Header->addVal($nshttp.'headerName',
		Cognition::RDFModel::Value->new($nshttpH.lc($header), 1));
	my $val = Cognition::RDFModel::Value->new($value);
	$Header->addVal($nshttp.'fieldValue', $val);

	if ($header =~ /content-length/i)
	{
		$val->setDatatype('http://www.w3.org/2001/XMLSchema#nonNegitiveInteger');
	}
	elsif ($header =~ /(\bdate$)|(^last-modified$)|(^if-modified-since$)/i)
	{
		my $dt   = Cognition::uF::datetime($value);
		my $val2 = Cognition::RDFModel::Value->new($dt);
		$Header->addVal($nshttp.'fieldValue', $val2);
	}
	elsif ($header =~ /link/i)
	{
		my $h2 = $value;
		my $uri = undef;
		if ($h2 =~ /^(\s*<([^>]*)>\s*)/)
		{
			$uri = $2;
			$h2  = substr($value, length($1));
		}
		else
		{
			next;
		}
		
		my $ParamList = Cognition::RDFModel::Subject->new(
			Cognition::HTMLParser::Utils::BNodeURI(undef, 'ParamList'));
		$ParamList->addVal($rdftype,
			Cognition::RDFModel::Value->new($nsrdf.'List', 1));

		my $HeaderElement = Cognition::RDFModel::Subject->new(
			Cognition::HTMLParser::Utils::BNodeURI(undef, 'HeaderElement'));
		$HeaderElement->addVal($rdftype,
			Cognition::RDFModel::Value->new($nshttp.'HeaderElement', 1));
		$HeaderElement->addVal($nshttp.'elementName',
			Cognition::RDFModel::Value->new($uri));
		$HeaderElement->addVal($nshttp.'params',
			Cognition::RDFModel::Value->new($ParamList->uri, 1));

		my $HeaderElementList = Cognition::RDFModel::Subject->new(
			Cognition::HTMLParser::Utils::BNodeURI(undef, 'HeaderElementList'));
		$HeaderElementList->addVal($rdftype,
			Cognition::RDFModel::Value->new($nsrdf.'List', 1));
		$HeaderElementList->addVal($nsrdf.'_1',
			Cognition::RDFModel::Value->new($HeaderElement->uri, 1));

		$Header->addVal($nshttp.'headerElements',
			Cognition::RDFModel::Value->new($HeaderElementList->uri, 1));
		
		$uri = $this->uri($uri);
		my $j = 1;
		while ($h2 =~ /^(\s*\;\s*(\/|[a-z0-9-]+)\s*\=\s*("[^"]*"|'[^']*'|[^\s\'\"\;\,]+)\s*)/i)
		{
			$h2 = substr($h2, length($1));
			my $key = $2;
			my $val = $3;

			$val =~ s/(^"|"$)//g if ($val =~ /^".*"$/);
			$val =~ s/(^'|'$)//g if ($val =~ /^'.*'$/ && $3 !~ /^".*"$/);

			my $Param = Cognition::RDFModel::Subject->new(
				Cognition::HTMLParser::Utils::BNodeURI(undef, 'Param'));
			$Param->addVal($rdftype, Cognition::RDFModel::Value->new($nshttp.'Param', 1));
			$Param->addVal($nshttp.'paramName', Cognition::RDFModel::Value->new($key));
			$Param->addVal($nshttp.'paramValue', Cognition::RDFModel::Value->new($val));
			$ParamList->addVal('http://buzzword.org.uk/rdf/list-items/_'.$j, Cognition::RDFModel::Value->new($Param->uri, 1));
			$this->mergeSubject($Param);
			$j++;
			
			if (lc($key) eq 'rel')
			{
				my $Object  = Cognition::RDFModel::Value->new($uri, 1);

				$val =~ s/(^\s+)|(\s+$)//g;
				$val =~ s/\s+/ /g;
				
				my @rels = split / /, $val;
				foreach my $rel (@rels)
				{
					$Subject->addVal(relationship($rel), $Object);

					$this->{Profiles}->{$uri}
						if (lc($rel) eq 'profile');
				}
			}

			elsif (lc($key) eq 'rev')
			{
				my $Object  = Cognition::RDFModel::Subject->new($uri);

				$val =~ s/(^\s+)|(\s+$)//g;
				$val =~ s/\s+/ /g;
				
				my @rels = split / /, $val;
				foreach my $rel (@rels)
				{
					print "[[[$rel]]]\n";
					$Object->addVal(relationship($rel),
						Cognition::RDFModel::Value->new($Subject->uri, 1));
					$this->mergeSubject($Object);
				}
			}
		}
		
		$this->mergeSubject($HeaderElementList);
		$this->mergeSubject($HeaderElement);
		$this->mergeSubject($ParamList);
	}

	$this->mergeSubject($Header);
	
	return $Header->uri;
}

1;
